@@ -19,6 +19,7 @@ import select
 
 from MAVProxy.modules.lib import textconsole
 from MAVProxy.modules.lib import mp_settings
+from MAVProxy.modules.lib import rline
 
 class MPStatus(object):
     '''hold status information about the mavproxy'''
@@ -38,13 +39,11 @@ class MPStatus(object):
         self.wp_op = None
         self.wp_save_filename = None
         self.wploader = mavwp.MAVWPLoader()
-        self.fenceloader = mavwp.MAVFenceLoader()
         self.loading_waypoints = False
         self.loading_waypoint_lasttime = time.time()
         self.mav_error = 0
         self.target_system = 1
         self.target_component = 1
-        self.rallyloader = mavwp.MAVRallyLoader(self.target_system, self.target_component)
         self.speech = None
         self.altitude = 0
         self.last_altitude_announce = 0.0
@@ -52,6 +51,7 @@ class MPStatus(object):
         self.last_battery_announce = 0
         self.last_avionics_battery_announce = 0
         self.battery_level = -1
+        self.voltage_level = -1
         self.avionics_battery_level = -1
         self.last_waypoint = 0
         self.exit = False
@@ -67,9 +67,6 @@ class MPStatus(object):
         self.last_apm_msg = None
         self.last_apm_msg_time = 0
         self.highest_msec = 0
-        self.fence_enabled = False
-        self.last_fence_breach = 0
-        self.last_fence_status = 0
         self.have_gps_lock = False
         self.lost_gps_lock = False
         self.last_gps_lock = 0
@@ -113,7 +110,6 @@ class MPState(object):
             [ ('link', int, 1),
               ('altreadout', int, 10),
               ('distreadout', int, 200),
-              ('battreadout', int, 0),
               ('heartbeat', int, 1),
               ('numcells', int, 1),
               ('speech', int, 0),
@@ -171,22 +167,6 @@ def get_usec():
     '''time since 1970 in microseconds'''
     return int(time.time() * 1.0e6)
 
-class rline(object):
-    '''async readline abstraction'''
-    def __init__(self, prompt):
-        import threading
-        self.prompt = prompt
-        self.line = None
-        try:
-            import readline
-        except Exception:
-            pass
-
-    def set_prompt(self, prompt):
-        if prompt != self.prompt:
-            self.prompt = prompt
-            sys.stdout.write(prompt)
-
 def say(text, priority='important'):
     '''speak some text'''
     ''' http://cvs.freebsoft.org/doc/speechd/ssip.html see 4.3.1 for priorities'''
@@ -304,7 +284,7 @@ def cmd_accelcal(args):
     while count < 6:
         m = mav.recv_match(type='STATUSTEXT', blocking=True)
         text = str(m.text)
-        if not text.startswith('Place APM'):
+        if not text.startswith('Place '):
             continue
         # wait for user to hit enter
         mpstate.rl.line = None
@@ -316,6 +296,29 @@ def cmd_accelcal(args):
         mav.mav.command_ack_send(count, 1)
 
 
+def cmd_compassmot(args):
+    '''do a compass/motor interference calibration'''
+    mav = mpstate.master()
+    print("compassmot starting")
+    mav.mav.command_long_send(mav.target_system, mav.target_component,
+                              mavutil.mavlink.MAV_CMD_PREFLIGHT_CALIBRATION, 0,
+                              0, 0, 0, 0, 0, 1, 0)
+    mpstate.rl.line = None
+    while True:
+        m = mav.recv_match(type=['COMMAND_ACK','COMPASSMOT_STATUS'], blocking=False)
+        if m is not None:
+            print(m)
+            if m.get_type() == 'COMMAND_ACK':
+                break
+        if mpstate.rl.line is not None:
+            # user has hit enter, stop the process
+            mav.mav.command_ack_send(0, 1)
+            break
+        time.sleep(0.01)
+    print("compassmot done")
+
+
+
 def cmd_reboot(args):
     '''reboot autopilot'''
     mpstate.master().reboot_autopilot()
@@ -580,273 +583,6 @@ def cmd_script(args):
 
     run_script(args[0])
 
-
-def fetch_fence_point(i):
-    '''fetch one fence point'''
-    mpstate.master().mav.fence_fetch_point_send(mpstate.status.target_system,
-                                                mpstate.status.target_component, i)
-    tstart = time.time()
-    p = None
-    while time.time() - tstart < 1:
-        p = mpstate.master().recv_match(type='FENCE_POINT', blocking=False)
-        if p is not None:
-            break
-        time.sleep(0.1)
-        continue
-    if p is None:
-        mpstate.console.error("Failed to fetch point %u" % i)
-        return None
-    return p
-
-
-
-def send_fence():
-    '''send fence points from fenceloader'''
-    # must disable geo-fencing when loading
-    action = get_mav_param('FENCE_ACTION', mavutil.mavlink.FENCE_ACTION_NONE)
-    param_set('FENCE_ACTION', mavutil.mavlink.FENCE_ACTION_NONE)
-    param_set('FENCE_TOTAL', mpstate.status.fenceloader.count())
-    for i in range(mpstate.status.fenceloader.count()):
-        p = mpstate.status.fenceloader.point(i)
-        mpstate.master().mav.send(p)
-        p2 = fetch_fence_point(i)
-        if p2 is None:
-            param_set('FENCE_ACTION', action)
-            return
-        if (p.idx != p2.idx or
-            abs(p.lat - p2.lat) >= 0.00003 or
-            abs(p.lng - p2.lng) >= 0.00003):
-            print("Failed to send fence point %u" % i)
-            param_set('FENCE_ACTION', action)
-            return
-    param_set('FENCE_ACTION', action)
-
-def load_fence(filename):
-    '''load fence points from a file'''
-    try:
-        mpstate.status.fenceloader.target_system = mpstate.status.target_system
-        mpstate.status.fenceloader.target_component = mpstate.status.target_component
-        mpstate.status.fenceloader.load(filename)
-    except Exception, msg:
-        print("Unable to load %s - %s" % (filename, msg))
-        return
-    print("Loaded %u geo-fence points from %s" % (mpstate.status.fenceloader.count(), filename))
-    send_fence()
-
-
-def list_fence(filename):
-    '''list fence points, optionally saving to a file'''
-
-    mpstate.status.fenceloader.clear()
-    count = get_mav_param('FENCE_TOTAL', 0)
-    if count == 0:
-        print("No geo-fence points")
-        return
-    for i in range(int(count)):
-        p = fetch_fence_point(i)
-        if p is None:
-            return
-        mpstate.status.fenceloader.add(p)
-
-    if filename is not None:
-        try:
-            mpstate.status.fenceloader.save(filename)
-        except Exception, msg:
-            print("Unable to save %s - %s" % (filename, msg))
-            return
-        print("Saved %u geo-fence points to %s" % (mpstate.status.fenceloader.count(), filename))
-    else:
-        for i in range(mpstate.status.fenceloader.count()):
-            p = mpstate.status.fenceloader.point(i)
-            mpstate.console.writeln("lat=%f lng=%f" % (p.lat, p.lng))
-    if mpstate.status.logdir != None:
-        fencetxt = os.path.join(mpstate.status.logdir, 'fence.txt')
-        mpstate.status.fenceloader.save(fencetxt)
-        print("Saved fence to %s" % fencetxt)
-
-
-def fence_draw_callback(points):
-    '''callback from drawing a fence'''
-    if len(points) < 3:
-        return
-    from MAVProxy.modules.lib import mp_util
-    mpstate.status.fenceloader.clear()
-    mpstate.status.fenceloader.target_system = mpstate.status.target_system
-    mpstate.status.fenceloader.target_component = mpstate.status.target_component
-    bounds = mp_util.polygon_bounds(points)
-    (lat, lon, width, height) = bounds
-    center = (lat+width/2, lon+height/2)
-    mpstate.status.fenceloader.add_latlon(center[0], center[1])
-    for p in points:
-        mpstate.status.fenceloader.add_latlon(p[0], p[1])
-    # close it
-    mpstate.status.fenceloader.add_latlon(points[0][0], points[0][1])
-    send_fence()
-
-def cmd_fence(args):
-    '''geo-fence commands'''
-    if len(args) < 1:
-        print("usage: fence <list|load|save|clear|draw>")
-        return
-
-    if args[0] == "load":
-        if len(args) != 2:
-            print("usage: fence load <filename>")
-            return
-        load_fence(args[1])
-    elif args[0] == "list":
-        list_fence(None)
-    elif args[0] == "save":
-        if len(args) != 2:
-            print("usage: fence save <filename>")
-            return
-        list_fence(args[1])
-    elif args[0] == "show":
-        if len(args) != 2:
-            print("usage: fence show <filename>")
-            return
-        mpstate.status.fenceloader.load(args[1])
-    elif args[0] == "draw":
-        if not 'draw_lines' in mpstate.map_functions:
-            print("No map drawing available")
-            return        
-        mpstate.map_functions['draw_lines'](fence_draw_callback)
-        print("Drawing fence on map")
-    elif args[0] == "clear":
-        param_set('FENCE_TOTAL', 0)
-    else:
-        print("Usage: fence <list|load|save|show|clear|draw>")
-
-
-def fetch_rally_point(i):
-    '''fetch one rally point'''
-    mpstate.master().mav.rally_fetch_point_send(mpstate.status.target_system,
-                                                mpstate.status.target_component, i)
-    tstart = time.time()
-    p = None
-    while time.time() - tstart < 1:
-        p = mpstate.master().recv_match(type='RALLY_POINT', blocking=False)
-        if p is not None:
-            break
-        time.sleep(0.1)
-        continue
-    if p is None:
-        mpstate.console.error("Failed to fetch rally point %u" % i)
-        return None
-    return p
-
-def send_rally_points():
-    '''send rally points from fenceloader'''
-    param_set('RALLY_TOTAL', mpstate.status.rallyloader.rally_count())
-    for i in range(mpstate.status.rallyloader.rally_count()):
-        p = mpstate.status.rallyloader.rally_point(i)
-        mpstate.master().mav.send(p)
-        
-        #Don't know why this check was being done, but not discounting yet...
-        #Will remove before committing if I don't discover why.
-        #p2 = fetch_rally_point(i)
-        #
-        #if (p.idx != p2.idx or
-        #    abs(p.lat - p2.lat) >= 0.00003 or
-        #    abs(p.lng - p2.lng) >= 0.00003):
-        #    print("Failed to send fence point %u" % i)
-        #    param_set('FENCE_ACTION', action)
-        #    return
-
-    #TODO rally land points
-
-def list_rally_points():
-    mpstate.status.rallyloader.clear()
-    rally_count = get_mav_param('RALLY_TOTAL', 0)
-    if rally_count == 0:
-        print("No rally points")
-        return
-    for i in range(int(rally_count)):
-        p = fetch_rally_point(i)
-        if p is None:
-            return
-        mpstate.status.rallyloader.append_rally_point(p)
-
-    #TODO: rally_land points
-
-    for i in range(mpstate.status.rallyloader.rally_count()):
-        p = mpstate.status.rallyloader.rally_point(i)
-        mpstate.console.writeln("lat=%f lng=%f alt=%f break_alt=%f land_dir=%f" % (p.lat * 1e-7, p.lng * 1e-7, p.alt, p.break_alt, p.land_dir))
-
-    #TODO: rally_land points
-
-def cmd_rally(args):
-    '''rally point commands'''
-    #TODO: add_land arg
-    if(len(args) < 1):
-        print("Usage: rally <list|load|save|add|clear>")
-        return
-
-    elif(args[0] == "add"):
-        if (len(args) < 2):
-            print("Usage: rally add ALT <BREAK_ALT> <LAND_HDG>")
-            return
-
-        if (mpstate.status.rallyloader.rally_count() > 4):
-            print ("Only 5 rally points possible per flight plan.")
-            return
-
-        try:
-            latlon = mpstate.map_state.click_position
-        except Exception:
-            print("No map available")
-            return
-        if latlon is None:
-            print("No map click position available")
-            return
-
-        alt = float(args[1]);
-        break_alt = 0.0
-        land_hdg = 0.0;
-        if (len(args) > 2):
-            break_alt = float(args[2]);
-        if (len(args) > 3):
-            land_hdg = float(args[3]);
-
-        mpstate.status.rallyloader.create_and_append_rally_point(latlon[0] * 1e7, latlon[1] * 1e7, alt, break_alt, land_hdg, 0)
-
-        send_rally_points();
-
-        print("Added Rally point at %s %f" % (str(latlon), alt))
-    
-    elif args[0] == "clear":
-        mpstate.status.rallyloader.clear()
-        param_set('RALLY_TOTAL', 0)
-
-    elif(args[0] == "list"):
-        list_rally_points()
-
-    elif(args[0] == "load"):
-        if (len(args) < 2):
-            print("Usage: rally load filename")
-
-        try:
-            mpstate.status.rallyloader.load(args[1])
-        except Exception, msg:
-            print("Unable to load %s - %s" % (args[1], msg))
-            return
-    
-        send_rally_points()
-
-        print("Loaded %u rally points from %s" % (mpstate.status.rallyloader.rally_count(), args[1]))
-
-    elif(args[0] == "save"):
-        if (len(args) < 2):
-            print("Usage: rally save filename");
-
-        mpstate.status.rallyloader.save(args[1]);
-
-        print "Saved rally file ", args[1];
-
-    else:
-        #TODO: add_land arg
-        print("Usage: rally <list|load|save|add|clear>")
-
 def param_set(name, value, retries=3):
     '''set a parameter'''
     name = name.upper()
@@ -1195,6 +931,9 @@ def cmd_module(args):
                 return
             except ImportError, msg:
                 ex = msg
+                if mpstate.settings.moddebug > 1:
+                    import traceback
+                    print traceback.format_exc()
         print("Failed to load module: %s" % ex)
     elif args[0] == "reload":
         if len(args) < 2:
@@ -1387,8 +1126,6 @@ command_map = {
     'rc'      : (cmd_rc,       'override a RC channel value'),
     'wp'      : (cmd_wp,       'waypoint management'),
     'script'  : (cmd_script,   'run a script of MAVProxy commands'),
-    'fence'   : (cmd_fence,    'geo-fence management'),
-    'rally'   : (cmd_rally,    'rally point management'),
     'param'   : (cmd_param,    'manage APM parameters'),
     'tuneopt' : (cmd_tuneopt,  'Select option for Tune Pot on Channel 6 (quadcopter only)'),
     'setup'   : (cmd_setup,    'go into setup mode'),
@@ -1399,6 +1136,7 @@ command_map = {
     'ground'  : (cmd_ground,   'do a ground start'),
     'level'   : (cmd_level,    'set level on a multicopter'),
     'accelcal': (cmd_accelcal, 'do 3D accelerometer calibration'),
+    'compassmot': (cmd_compassmot, 'do compass/motor interference calibration'),
     'calpress': (cmd_calpressure,'calibrate pressure sensors'),
     'loiter'  : (cmd_loiter,   'set LOITER mode'),
     'rtl'     : (cmd_rtl,      'set RTL mode'),
@@ -1555,6 +1293,7 @@ def battery_update(SYS_STATUS):
 
     # main flight battery
     mpstate.status.battery_level = SYS_STATUS.battery_remaining
+    mpstate.status.voltage_level = SYS_STATUS.voltage_battery
 
     # avionics battery
     if not 'AP_ADC' in mpstate.status.msgs:
@@ -1572,22 +1311,26 @@ def battery_update(SYS_STATUS):
     else:
         mpstate.status.avionics_battery_level = (95*mpstate.status.avionics_battery_level + 5*avionics_battery_level)/100
 
-
-
 def battery_report():
-    '''report battery level'''
-    if int(mpstate.settings.battreadout) == 0:
-        return
+    batt_mon = mpstate.mav_param.get('BATT_MONITOR',0)
 
-    mpstate.console.set_status('Battery', 'Battery: %u' % mpstate.status.battery_level, row=0)
+    #report voltage level only 
+    if batt_mon == 3:
+        mpstate.console.set_status('Battery', 'Batt: %.2fV' % (float(mpstate.status.voltage_level) / 1000.0), row=0)
+    elif batt_mon == 4:
+        mpstate.console.set_status('Battery', 'Batt: %u%%/%.2fV' % (mpstate.status.battery_level, (float(mpstate.status.voltage_level) / 1000.0)), row=0)
 
-    rbattery_level = int((mpstate.status.battery_level+5)/10)*10
+        rbattery_level = int((mpstate.status.battery_level+5)/10)*10
 
-    if rbattery_level != mpstate.status.last_battery_announce:
-        say("Flight battery %u percent" % rbattery_level, priority='notification')
-        mpstate.status.last_battery_announce = rbattery_level
-    if rbattery_level <= 20:
-        say("Flight battery warning")
+        if rbattery_level != mpstate.status.last_battery_announce:
+            say("Flight battery %u percent" % rbattery_level, priority='notification')
+            mpstate.status.last_battery_announce = rbattery_level
+        if rbattery_level <= 20:
+            say("Flight battery warning")
+
+    else:
+        #clear battery status
+        mpstate.console.set_status('Battery')
 
     # avionics battery reporting disabled for now
     return
@@ -1843,22 +1586,13 @@ def master_callback(m, master):
             if rounded_dist != 0:
                 say("%u" % rounded_dist, priority="progress")
             mpstate.status.last_distance_announce = rounded_dist
-
-    elif mtype == "FENCE_STATUS":
-        if not mpstate.status.fence_enabled:
-            mpstate.status.fence_enabled = True
-            say("fence enabled")
-        if mpstate.status.last_fence_breach != m.breach_time:
-            say("fence breach")
-        if mpstate.status.last_fence_status != m.breach_status:
-            if m.breach_status == mavutil.mavlink.FENCE_BREACH_NONE:
-                say("fence OK")
-        mpstate.status.last_fence_breach = m.breach_time
-        mpstate.status.last_fence_status = m.breach_status
-
+    
     elif mtype == "GLOBAL_POSITION_INT":
         report_altitude(m.relative_alt*0.001)
 
+    elif mtype == "COMPASSMOT_STATUS":
+        print(m)
+
     elif mtype == "BAD_DATA":
         if mpstate.settings.shownoise and mavutil.all_printable(m.data):
             mpstate.console.write(str(m.data), bg='red')
@@ -1907,7 +1641,14 @@ def process_master(m):
     try:
         s = m.recv(16*1024)
     except Exception:
+        time.sleep(0.1)
         return
+    # prevent a dead serial port from causing the CPU to spin. The user hitting enter will
+    # cause it to try and reconnect
+    if len(s) == 0:
+        time.sleep(0.1)
+        return
+    
     if mpstate.logqueue_raw:
         mpstate.logqueue_raw.put(str(s))
 
@@ -2157,7 +1898,7 @@ def main_loop():
         for fd in mpstate.select_extra:
             rin.append(fd)
         try:
-            (rin, win, xin) = select.select(rin, [], [], 0.0001)
+            (rin, win, xin) = select.select(rin, [], [], 0.01)
         except select.error:
             continue
 
@@ -2220,9 +1961,7 @@ def run_script(scriptfile):
         process_stdin(line)
     f.close()
 
-
 if __name__ == '__main__':
-
     from optparse import OptionParser
     parser = OptionParser("mavproxy.py [options]")
 
@@ -2270,6 +2009,7 @@ if __name__ == '__main__':
     parser.add_option("--nowait", action='store_true', default=False, help="don't wait for HEARTBEAT on startup")
     parser.add_option("--continue", dest='continue_mode', action='store_true', default=False, help="continue logs")
     parser.add_option("--dialect",  default="ardupilotmega", help="MAVLink dialect")
+    parser.add_option("--rtscts",  action='store_true', help="enable hardware RTS/CTS flow control")
 
     (opts, args) = parser.parse_args()
 
@@ -2319,6 +2059,8 @@ Auto-detected serial ports are:
         m.mav.set_callback(master_callback, m)
         if hasattr(m.mav, 'set_send_callback'):
             m.mav.set_send_callback(master_send_callback, m)
+        if opts.rtscts:
+            m.set_rtscts(True)
         m.linknum = len(mpstate.mav_master)
         m.linkerror = False
         m.link_delayed = False
@@ -2341,10 +2083,6 @@ Auto-detected serial ports are:
         if os.path.exists(waytxt):
             mpstate.status.wploader.load(waytxt)
             print("Loaded waypoints from %s" % waytxt)
-        fencetxt = os.path.join(mpstate.status.logdir, 'fence.txt')
-        if os.path.exists(fencetxt):
-            mpstate.status.fenceloader.load(fencetxt)
-            print("Loaded fence from %s" % fencetxt)
 
     # open any mavlink UDP ports
     for p in opts.output:
@@ -2369,7 +2107,7 @@ Auto-detected serial ports are:
         mpstate.override_period = mavutil.periodic_event(1)
     heartbeat_check_period = mavutil.periodic_event(0.33)
 
-    mpstate.rl = rline("MAV> ")
+    mpstate.rl = rline.rline("MAV> ")
     if opts.setup:
         mpstate.rl.set_prompt("")
 
@@ -2387,7 +2125,7 @@ Auto-detected serial ports are:
 
     if not opts.setup:
         # some core functionality is in modules
-        standard_modules = ['log']
+        standard_modules = ['log','rally','fence']
         for m in standard_modules:
             process_stdin('module load %s' % m)
 
